#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ###########################################################
# Minimal-Demonstrator für die OSMscr2-Klasse.
#
# Detlev Ahlgrimm, 2017


import wx               # python-wxWidgets-2.8.12.1-10.4.1.x86_64
import os
import time
from OSMscr3 import OSMscr3
from OSMhelper import OSMhelper

try:
  from Database4wd        import Database
  DATABASE_FILE="/home/dede/daten/wardriving.sqlite"
  wardriving_db_found=True
except:
  wardriving_db_found=False

VERSION="1.0"

HOME_DIR=os.path.expanduser('~')
SOURCE={
  "OSM"   : { "tile_url"  : "http://tile.openstreetmap.org/{z}/{x}/{y}.png",
              "tile_size" : 256,
              "max_zoom"  : 19,
              "cache_dir" : os.path.join(HOME_DIR, "wd", "osm")
            },
  "OCM"   : { "tile_url"  : "https://tile.thunderforest.com/cycle/{z}/{x}/{y}.png?apikey=0ab849f67c69480eb29423a2fb809625",
              "tile_size" : 256,
              "max_zoom"  : 18,
              "cache_dir" : os.path.join(HOME_DIR, "wd", "ocm")
            },
        }
SERVER="OSM"

class MapWindow(wx.Window):
  def __init__(self, parent, centerLL, zoom, border, title, size):
    wx.Window.__init__(self, parent)
    self.parent=parent
    self.centerLL=centerLL
    self.zoom=zoom
    self.border=border
    self.Bind(wx.EVT_PAINT,         self.onPaint)
    self.Bind(wx.EVT_LEFT_DOWN,     self.onLeftDown)
    self.Bind(wx.EVT_LEFT_UP,       self.onLeftUp)
    self.Bind(wx.EVT_RIGHT_DOWN,    self.onRightDown)
    self.Bind(wx.EVT_MOTION,        self.onMotion)
    self.Bind(wx.EVT_MOUSEWHEEL,    self.onMouseWheel)
    self.Bind(wx.EVT_SIZE,          self.onSize)
    self.Bind(wx.EVT_TIMER,         self.onTimer)
    #self.osm=OSMscr3(self.parent.GetSize(), SOURCE[SERVER], centerLL, self.zoom, self.border)
    self.osm=OSMscr3(SOURCE[SERVER])
    self.dc_sizeXY=self.parent.GetSize()
    print "__init__, self.dc_sizeXY", self.dc_sizeXY
    self.timer_interval=100
    self.timer=wx.Timer(self)
    self.refresh_needed=False
    self.leftDown=False
    self.autofollow=False


    self.handle=self.osm.setup(self.dc_sizeXY, centerLL, self.zoom, self.border)
    self.handle=self.osm.newMap(centerLL)
    self.hlp=OSMhelper()
    #self.tracks, self.waypoints=self.hlp.loadGPX("/home/dede/wd/xml_tracker/2016/wardriving/2016_07_10_11_35_27.gpx")
    #for trk in self.tracks:
    #  h=self.osm.loadTrack(trk)
    #  print "handle=", h
    #self.tracks, self.waypoints=self.hlp.loadGPX("/home/dede/wd/xml_tracker/2016/2016_09_05_11_21_22 - Bauhaus.gpx")
    #self.tracks, self.waypoints=self.hlp.loadGPX("/home/dede/GPS-Tracks/Bauhaus.gpx")
    #self.tracks, self.waypoints=self.hlp.loadGPX("/home/dede/wd/xml_tracker/2017_05_21_10_12_11 - Mohrkirch, Atzbüll, Neukirchen.gpx")
    self.tracks, self.waypoints=self.hlp.loadGPX("/home/dede/GenWesten.gpx")

    if wardriving_db_found:
      self.db=Database(DATABASE_FILE)

      aps=self.db.getAllAPs()
      self.point_list=list()
      for ap in aps:
        #      0    1     2     3       4       5       6             7           8
        # ap=(lat, lon, bssid, name, channel, crypto, calc_points, first_seen, last_seen)
        if ap[3] not in ["Vodafone Hotspot", "Vodafone Homespot", "Telekom_FON"]:
          data=(ap[2], ap[3], int(ap[4]), ap[5], int(ap[6]), ap[7], ap[8], float(ap[0]), float(ap[1]))
          if ap[5]=="None":
            self.point_list.append(((float(ap[0]), float(ap[1])), 5, (0, 255, 0), (0, data)))
          else:
            self.point_list.append(((float(ap[0]), float(ap[1])), 3, (255, 0, 0), (0, data)))
      self.wardriving_point_handle=self.osm.loadPoints(self.point_list)


    if False:
      self.tracks=[[  ((54.805072420525505, 9.524819254875183), 1), 
                      ((54.803767644743864, 9.52488362789154), 2), 
                      #((54.80508324194943, 9.52451080083847), 3),
                      ((54.80512034395235, 9.522829055786133), 4),
                      ((54.803792380221516, 9.523357450962067), 5),
                      ((54.80477096540826, 9.524601995944977), 6),
                      ((54.804568448143606, 9.525280594825745), 7) ]]

    for trk in self.tracks:
      h=self.osm.loadTrack(trk, (0, 0, 0), 2)
      print "handle=", h
      pts=list()
      for ll, ts in trk:
        pts.append((ll, 3, (0, 0, 255), ""))
      #self.osm.loadPoints(pts)

    t=[ ((54.804325735275626, 9.523773193359375), 1),
        ((54.814325735275626, 9.523773193359375), 2),
        ((54.814325735275626, 9.524773193359375), 3),
        ((54.804325735275626, 9.524773193359375), 4),
           ]
    #h=self.osm.loadTrack(t, (0, 0, 255), 4)

    pt1=((54.80542643417665, 9.525138437747955), 10, (0, 0, 255), (1, "blau", 66))
    pt2=((54.805060, 9.524878), 6, (0, 255, 0), "zuhause")
    pt3=((54.80482198104772, 9.525833129882812), 4, (0, 255, 0), "bei hausenummer 24")
    pt4=((54.80542798008099, 9.523778557777405), 4, (0, 255, 0), "links oben")
    pt5=((54.804648836797625, 9.52513575553894), 4, (0, 255, 0), "rechts unten")
    pt6=((54.804648836797625, 9.523778557777405), 4, (0, 255, 0), "links unten")
    pt7=((54.8046411071263, 9.524569809436798), 4, (0, 255, 0), "unten mittig")
    self.osm.loadPoints([pt1, pt2, pt3, pt4, pt5, pt6, pt7])
    #self.osm.loadPoints([pt7])

    self.osm.setZoom(self.zoom, (0, 0), self.handle)
    self.osm.setCenter(self.centerLL, self.handle)
    #self.osm.setCenter((54.8044803296276, 9.524041414260864), self.handle)

    self.dll=list()
    self.last_painted_handle=-1

    #self.osm.loadPoints([(centerLL, 5,  wx.Brush("RED"), (1, " point "))])
    self.timer.Start(self.timer_interval)
    self.Refresh()


  def onPaint(self, evt):
    new_bmp, (sx, sy)=self.osm.drawMap(self.handle)
    if new_bmp or self.last_painted_handle!=self.handle:
      self.last_painted_handle=self.handle
      (w, h, bmpdata), (sx, sy), self.refresh_needed=self.osm.getBitmap(self.handle)
      self.bmpbuf=wx.BitmapFromBuffer(w, h, bmpdata)
    dc=wx.PaintDC(self)
    dc.DrawBitmap(self.bmpbuf, sx, sy)

    #posXY=self.osm.getPixelForLatLon((54.806278218506, 9.523660540581), self.handle)
    dc.SetPen(wx.Pen("BLACK"))
    dc.SetBrush(wx.Brush("RED"))
    #dc.DrawCirclePoint(posXY, 5)
    dc.SetBrush(wx.Brush("WHITE"))
    dc.DrawCirclePoint((self.dc_sizeXY[0]/2, self.dc_sizeXY[1]/2), 5)

    
    return
    if new_bmp or self.last_painted_handle!=self.handle:
      self.last_painted_handle=self.handle
      # durch den Umweg über einen MemoryDC wird erreicht, dass eine
      # unveränderte Bitmap nur einmal konvertiert werden muss - und
      # dadurch die Systemlast bei "Shift" signifikant sinkt.
      imgbuf, (sx, sy), self.refresh_needed=self.osm.getBitmap(self.handle)
      w, h=imgbuf.size
      self.bmpbuf=wx.EmptyBitmapRGBA(w, h, 204, 204, 204, 1)
      mdc=wx.MemoryDC()
      print "new MemoryDC()"
      mdc.SelectObject(self.bmpbuf)
      mdc.DrawBitmap(wx.BitmapFromBuffer(w, h, imgbuf.tostring()), 0, 0)
      del mdc
    dc=wx.PaintDC(self)
    dc.DrawBitmap(self.bmpbuf, sx, sy)

    for xy in self.dll:
      dc.DrawLine(0, 0, xy[0], xy[1])


  def onLeftDown(self, evt):
    self.leftDown=True
    self.was_moved=False
    self.leftDownPosXY=evt.GetPosition()
    self.SetCursor(wx.StockCursor(wx.CURSOR_HAND))
    print "Klick:", self.osm.getLatLonForPixel(self.leftDownPosXY, self.handle)

  def onLeftUp(self, evt):
    self.leftDown=False
    self.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))
    if self.was_moved:
      self.osm.endMoveMap(self.handle)
      self.Refresh()

  def onRightDown(self, evt):
    print self.handle
    #self.handle^=1
    #print self.handle
    #self.Refresh()
    for idx in xrange(len(self.tracks[0])-1):
      ll, ts=self.tracks[0][idx]
      llp, ts=self.tracks[0][idx+1]
      self.osm.setCenter(llp, self.handle^1)
      self.osm.drawMap(self.handle^1)
      self.scrollMapToPoint(ll, llp, 1, 0.01, self.handle)
      self.handle^=1
    evt.Skip(True)

  def onMotion(self, evt):
    self.dll=list()
    refresh=False
    data=self.osm.getDataForPoint(evt.GetPosition(), self.handle)      
    if len(data)>0:
      print data
#      for xy, d in data:
#        self.dll.append(xy)
#        refresh=True
    if self.leftDown:
      self.was_moved=True
      scroll_distXY=self.leftDownPosXY-evt.GetPosition()
      self.osm.doMoveMap(scroll_distXY, self.handle)
      refresh=True
    if refresh:
      self.Refresh()

  def onMouseWheel(self, evt):
    zoom_old=self.zoom
    if evt.GetWheelRotation()>0:
      self.zoom=min(SOURCE[SERVER]["max_zoom"], self.zoom+1)
    else:
      self.zoom=max(0, self.zoom-1)
    if self.zoom!=zoom_old:
      self.osm.setZoom(self.zoom, evt.GetPosition(), self.handle)
      self.Refresh()

  def onSize(self, evt):
    try:
      self.dc_sizeXY=self.GetSize()
      print "onSize, self.dc_sizeXY", self.dc_sizeXY
      self.osm.setSize(self.dc_sizeXY)
      #self.osm.setSize(self.GetSize())
    except:
      pass  # der allererste EVT_SIZE kommt, bevor self.osm existiert

  def onTimer(self, evt):
    if self.refresh_needed and not self.autofollow:
      print "Refresh()"
      self.Refresh()

  # ######################################################################
  # Scrollt von der Geo-Koordinate "posFromLL" nach "posToLL" mit einer
  # Schrittgröße von "step_size" Pixeln und "delay" Sekunden Wartezeit
  # zwischen den Schritten.
  # Der Abstand der Koodinaten sollte in "self.border" passen, weil
  # zwischendurch keine Tiles nachgeladen werden.
  def scrollMapToPoint(self, posFromLL, posToLL, step_size, delay, handle=0):
    if posFromLL==posToLL:
      return
    p1XY=self.osm.getPixelForLatLon(posFromLL, handle)
    p2XY=self.osm.getPixelForLatLon(posToLL, handle)
    wXY=self.hlp.getLine(p1XY, p2XY)
    pcXY=wXY[0]
    self.autofollow=True
    for idx in range(1, len(wXY)-(step_size-1), step_size):
      d=(wXY[idx][0]-pcXY[0], wXY[idx][1]-pcXY[1])
      self.osm.doMoveMap(d, handle)
      self.Refresh()
      wx.Yield()
#      if self.esc:
#        break
      time.sleep(delay)
    self.osm.endMoveMap(handle)
    self.osm.setCenter(posToLL, handle)
    self.autofollow=False
    #self.Refresh()
    #wx.Yield()


class MapFrame(wx.Frame):
  def __init__(self, parent, centerLL, zoom, border, title, pos=wx.DefaultPosition, size=wx.DefaultSize):
    wx.Frame.__init__(self, None, wx.ID_ANY, title+" "+VERSION, pos=pos, size=size)
    win=MapWindow(self, centerLL, zoom, border, title, size)


if __name__=="__main__":
  app=wx.App(False)
  frame=MapFrame(None, (54.805060, 9.524878), 18, (2, 2), "wxOSMmini", size=(1024, 1100))
  frame.Show()
  app.MainLoop()
